% GL = Gamma_Nonlinear2( Vplus, VDC, Z0 )
%
% author:  Greg Durgin (1/26/2012)
%
% Calculates the incremental reflection coefficient on a transmission line
% with a non-linear load using the following definitions/inputs:
%
% GL - incremental load reflection coefficient (unitless)
% Vplus - new forward-traveling voltage amplitude striking the load side
% Z0 - intrinsic impedance (Ohms)
% VDC - existing DC voltage at the load from prior reflections (V)
% VI_curve - vector of polynomial coefficients determining VI curve in the
%    form I = a1*V^3+a2*V^2+a3*V+a4 
%
function GL = Gamma_Nonlinear2( Vplus, VDC, Z0 )
    Nmax = 50;          % maximum number of iterations for convergence
    n = 0;              % starting index/counter
    VI_curve = [ .15 -.12 +.025 0 ];     % polynomial of VI curve: a1*V^3+a2*V^2+a3*V+a4

% compute initial forward/backward voltages
    Vpall = (VDC + Current(VI_curve,VDC )* Z0)/2 + Vplus; % new Vplus
    Vmall = (VDC - Current(VI_curve,VDC )* Z0)/2;    % existing Vminus
    poly = VI_curve + [ 0 0 1 -2*Vpall]/Z0;    % nonlinear equation to solve
    Vnew = roots(poly);     % solve for new voltage
    
    if isreal(Vnew(1))  % find the real root (complex values discarded)
        Vdc = Vnew(1);
    elseif isreal(Vnew(2))
        Vdc = Vnew(2);
    elseif isreal (Vnew(3))
        Vdc = Vnew(3);
    else
        Vdc = VDC;
    end;
    
    Vmm = (Vdc - Current(VI_curve,Vdc)* Z0)/2 - Vmall;  % incremental V-
    if abs(Vplus) > 0
        GL = Vmm/Vplus;     % nonlinear reflection coefficient
    else
        GL = 0;             % no change
    end;

% Helper function that computes the VI-curve of the nonlinear device    
function I = Current( VI_curve, V )
   I = VI_curve(1)*V.^3 + VI_curve(2)*V.^2 + VI_curve(3)*V + VI_curve(4);