% Vout = NonlinearTlinePlot( T, Z0, Vs, Rs )
%
% author:  Greg Durgin (1/26/2012)
%
% Calculates, graphs, and returns the output voltage of a transmission line
% with an arbitrary DC source and a non-linear termination.  Variables in
% the function definition are given below:
%
% Vout - vector of values w.r.t. time of the output of the t-line
% T - transit time for the line (s)
% Z0 - intrinsic impedance (Ohms)
% Vs - Source DC voltage (V)
% Rs - Source Resistance (Ohms)
%
function Vout = NonlinearTlinePlot( T, Z0, Vs, Rs )

Tend = 250*T;               % maximum time duration to plot
N = round((Tend-T)/2/T);    % calculate number of iterations
GammaS = (Rs-Z0)/(Rs+Z0);   % source-side reflection coefficient
Vplus = Z0*Vs/(Z0+Rs);      % initial voltage amplitude launched
Vout = zeros(2*N+2,1);      % array for storing output voltages
VDC = 0;                    % starting condition for load
t = linspace(0,2*(N+1)*T,2*(N+1));      % time vector used for graphing

% Iterate through the reflections in units of 2T
for n = 1:N,                            % iterate through all reflections
    GammaL = Gamma_Nonlinear2(Vplus, VDC, Z0); 
    Vout(2*n:end) = Vout(2*n:end) + (1+GammaL)*Vplus;
    VDC = Vout(2*n);                    % new accumulated voltage
    Vneg = Vplus * GammaL;              % next load-reflected wave
    Vplus = Vneg * GammaS;              % next source-reflected wave
end;

% Plot the result and label accordingly
plot(t*1e9, Vout);
set(gca,'FontSize',14);
axis([0 50*T*1e9 min(Vout) max(Vout)+.05]);
xlabel('time (ns)');
ylabel('load voltage (V)');
title(sprintf('Output of Transmission Line with R_s=%2.0f \\Omega',Rs));

    
